/******************************************************************************
 * $Header: /CommonBe/agmsmith/Programming/AGMSRAMFileSystem/RCS/AGMSRAMFSExports.h,v 1.8 2002/07/05 18:21:43 agmsmith Exp $
 *
 * This header file defines the exported user level things from the
 * AGMSRAMFileSystem, mostly things for IOCtl operations.
 *
 * Copyright © 2001 by Alexander G. M. Smith.  See AGMSRAMFileSystem.c for
 * licensing details.
 *
 * $Log: AGMSRAMFSExports.h,v $
 * Revision 1.8  2002/07/05 18:21:43  agmsmith
 * Added case insensitivity flag for indices.
 *
 * Revision 1.7  2002/06/29 21:39:04  agmsmith
 * Added some new flags for fs_create_index.
 *
 * Revision 1.6  2002/04/28 20:31:18  agmsmith
 * Added a new flag for setting file system read only mode.
 *
 * Revision 1.5  2002/03/31 20:09:44  agmsmith
 * Added system for limiting the total file size, to avoid using
 * up too much memory and causing a kernel crash.
 *
 * Revision 1.4  2002/03/17 20:04:45  agmsmith
 * Now with a working llink function.
 *
 * Revision 1.3  2002/02/26 01:56:47  agmsmith
 * Added llink ioctl.
 *
 * Revision 1.2  2001/12/31 19:41:12  agmsmith
 * Added S_IFAGMS constant for identifying AGMS links, removed old
 * debug buffers stuff.
 *
 * Revision 1.1  2001/07/02 22:00:42  agmsmith
 * Initial revision
 */

#ifndef AGMSRAMFSEXPORTS_H
#define AGMSRAMFSEXPORTS_H 1

#include <Drivers.h>

#ifdef  __cplusplus
extern "C" {
#endif

/* New file type for use with rstat (read status) to identify AGMS links (Alias
Generated Morphing Symbolic links).  See /posix/sys/stat.h for the other ones
(these defines would move there if they ever became standard).  However, they
aren't actually used yet since that could cause problems with existing
software, so for now they appear as regular symbolic links with size 1 (AGMS
links) or 2 (fake symbolic links, used for the "..." and higher extra parent
directories, which should really be hard links like "." and ".." are). */

#define     S_IFAGMS       00000130000 /* AGMS link */
#define     S_ISAGMS(m)    (((m) & S_IFMT) == S_IFAGMS)


/* An extra option for changing file system properties in wfsstat. */

#define         WFSSTAT_USEFLAG_READONLY 0x0002


/* Some new flags for fs_create_index. */

#define		B_CREATE_INDEX_WITH_REBUILD         0x00000001
/* If specified as a flag to fs_create_index, the new index will be populated
with all the files which have the attribute matching the new index's name.
Well, almost all, ones that are too small for a fixed size (non-string)
attribute type will be ignored.  This can take a while. */

#define		B_CREATE_INDEX_MULTIPLE_KEYWORDS    0x00000002
/* Makes the new index a multiple keyword index.  When an attribute is added,
it is broken into keywords (defined as the words separated by control
characters or spaces) and each keyword is added separately.  Non-string
attributes are treated as arrays of key-things, where the number of elements is
the attribute size divided by the data type size. */

#define         B_CREATE_INDEX_CASE_INSENSITIVE     0x00000004
/* For string based indices, tells the system to ignore the case (upper case
and lower case are treated the same) of the letters.  Internally the index
does store the case, so when you do a query for a name, you see the results
with the original case used in the key strings. */


/* Custom IOCtl operations for AGMSRAMFileSystem. */

typedef enum ARFSIOCtlNumbersEnum
{
  ARFS_IOCTL_BEFORE_FIRST = B_DEVICE_OP_CODES_END + 233,
  ARFS_IOCTL_LLINK,
  ARFS_IOCTL_GET_MAX_TOTAL_FILE_SIZE,
  ARFS_IOCTL_SET_MAX_TOTAL_FILE_SIZE,
  ARFS_IOCTL_MAX
} ARFSIOCtlNumbers;


typedef struct ARFSLLinkIoctlStruct
{
  int   structureSize;
  char *newName;
  ino_t targetVNodeID;
} ARFSLLinkIoctlRecord, *ARFSLLinkIoctlPointer;


extern int llink (const char *name, const char *new_name);
extern int32 ARFS_GetMaxTotalFileSize (const char *FileName);
extern int32 ARFS_SetMaxTotalFileSize (const char *FileName, int32 MaxSize);



/******************************************************************************
 * The normal "link ()" call for making hard links (actually AGMS links in the
 * AGMSRAMFileSystem case) follows links so it doesn't work when you want to
 * make a link to a link.  Instead, use this function.  Makes a new link with
 * name new_name, targetting the existing file or even a symbolic or AGMS link
 * specified by name.  Should really be in a library, or even in the kernel,
 * but...  Just define AGMSRAMFS_LLINK_INCLUDE_CODE in one (only one) of your
 * source files if you want the code to be compiled.  Returns -1 on failure
 * (with the error code in errno), 0 on success.
 */

#ifdef AGMSRAMFS_LLINK_INCLUDE_CODE
int llink (const char *name, const char *new_name)
{
  ARFSLLinkIoctlRecord IOctlParameters;
  int                  NewDirFD = -1;
  struct stat          NewDirStat;
  char                 NewDirPath [PATH_MAX];
  status_t             ReturnCode = -1;
  char                *StringPntr;
  int                  TargetFD = -1;
  struct stat          TargetStat;

  if (name == NULL || new_name == NULL)
  {
    ReturnCode = EFAULT;
    goto ErrorExit;
  }

  /* Force the OS to load the file record (vnode) of the target thing and keep
  it in memory until this function is finished.  Done by opening the target as
  a file. */

  TargetFD = open (name, O_RDONLY | O_NOTRAVERSE);
  if (TargetFD < 0)
  {
    ReturnCode = errno;
    goto ErrorExit;
  }

  /* Put the name of the directory that the new name will be created in, into
  NewDirPath.  Leave StringPntr pointing to the start of the name portion of
  the new name. */

  strncpy (NewDirPath, new_name, sizeof (NewDirPath) - 1);
  NewDirPath[sizeof (NewDirPath) - 1] = 0;
  StringPntr = strrchr (NewDirPath, '/');
  if (StringPntr == NULL)
  {
    strcpy (NewDirPath, ".");
    StringPntr = (char *) new_name;
  }
  else
    *StringPntr++ = 0;

  /* Also open that directory so it won't disappear.  Yes, you can open
  directories (but you can't read data from them).  Allow symbolic link
  traversal in the directory name. */

  NewDirFD = open (NewDirPath, O_RDONLY);
  if (NewDirFD < 0)
  {
    ReturnCode = errno;
    goto ErrorExit;
  }

  /* Now get the vnodeid numbers to identify the directory and the thing.  Also
  make sure they are on the same file system. */

  if (fstat (TargetFD, &TargetStat))
  {
    ReturnCode = errno;
    goto ErrorExit;
  }
  if (fstat (NewDirFD, &NewDirStat))
  {
    ReturnCode = errno;
    goto ErrorExit;
  }
  if (TargetStat.st_dev != NewDirStat.st_dev)
  {
    ReturnCode = EXDEV;
    goto ErrorExit;
  }
  if (!S_ISDIR (NewDirStat.st_mode))
  {
    ReturnCode = ENOTDIR;
    goto ErrorExit;
  }

  /* Set up the ioctl operation to do the actual work. */

  IOctlParameters.structureSize = sizeof (IOctlParameters);
  IOctlParameters.newName = StringPntr;
  IOctlParameters.targetVNodeID = TargetStat.st_ino;
  if (ioctl (NewDirFD, ARFS_IOCTL_LLINK, &IOctlParameters))
    ReturnCode = errno;
  else
    ReturnCode = 0;

ErrorExit:
  if (TargetFD >= 0) close (TargetFD);
  if (NewDirFD >= 0) close (NewDirFD);
  if (ReturnCode != 0)
  {
    errno = ReturnCode;
    return -1;
  }
  return 0;
}
#endif /* AGMSRAMFS_LLINK_INCLUDE_CODE */



/******************************************************************************
 * These ioctl interface functions let you get and set the total maximum file
 * size.  Define AGMSRAMFS_MAXTOTALFILESIZE_INCLUDE_CODE in one (only one) of
 * your source files if you want this code to be compiled.  The
 * AGMSRAMFileSystem will keep track of the total of file sizes of all files
 * and cut off write operations that would make the total bigger than this
 * limit.  It is used to prevent the file system from using up all memory,
 * which crashes the BeOS kernel.  Typically the maximum should be between 1
 * and at most 800 megabytes, to avoid killing the kernel.  In practice, a good
 * maximum is your total real memory size minus 50 megabytes.  The file name is
 * the name of any file or directory on the AGMSRAMFileSystem, it doesn't
 * matter which one.  Returns the size/zero on success, returns -1 on failure
 * and sets the errno code.
 */

#ifdef AGMSRAMFS_MAXTOTALFILESIZE_INCLUDE_CODE
int32 ARFS_GetMaxTotalFileSize (const char *FileName)
{
  status_t ReturnCode = -1;
  int      TargetFD = -1;
  int32    Value;

  if (FileName == NULL)
  {
    ReturnCode = EFAULT;
    goto ErrorExit;
  }

  TargetFD = open (FileName, O_RDONLY | O_NOTRAVERSE);
  if (TargetFD < 0)
  {
    ReturnCode = errno;
    goto ErrorExit;
  }

  Value = 0;
  if (ioctl (TargetFD, ARFS_IOCTL_GET_MAX_TOTAL_FILE_SIZE, &Value))
    ReturnCode = errno;
  else
    ReturnCode = 0;

ErrorExit:
  if (TargetFD >= 0) close (TargetFD);
  if (ReturnCode != 0)
  {
    errno = ReturnCode;
    return -1;
  }
  return Value;
}



int32 ARFS_SetMaxTotalFileSize (const char *FileName, int32 MaxSize)
{
  status_t ReturnCode = -1;
  int      TargetFD = -1;

  if (FileName == NULL)
  {
    ReturnCode = EFAULT;
    goto ErrorExit;
  }

  TargetFD = open (FileName, O_RDONLY | O_NOTRAVERSE);
  if (TargetFD < 0)
  {
    ReturnCode = errno;
    goto ErrorExit;
  }

  if (ioctl (TargetFD, ARFS_IOCTL_SET_MAX_TOTAL_FILE_SIZE, &MaxSize))
    ReturnCode = errno;
  else
    ReturnCode = 0;

ErrorExit:
  if (TargetFD >= 0) close (TargetFD);
  if (ReturnCode != 0)
  {
    errno = ReturnCode;
    return -1;
  }
  return 0; /* Success. */
}
#endif /* AGMSRAMFS_MAXTOTALFILESIZE_INCLUDE_CODE */



#ifdef  __cplusplus
}
#endif

#endif /* AGMSRAMFSEXPORTS_H */
